#
# $Id: drmaa.pbs_drmaa.pyx 533 2007-12-22 15:25:42Z lukasz $
#
# Python bindings to FedStage DRMAA for PBS Pro
# Copyright (C) 2006-2007  Fedstage Systems Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

cimport drmaa.core as drmaa_pyrex

cdef extern from *:
	ctypedef int size_t

cdef extern from "stdlib.h":
	void *malloc( size_t size )
	void *calloc( size_t nmemb, size_t size )
	void free( void *ptr )
	void *realloc( void *ptr, size_t size )

cdef extern from "stdio.h":
	ctypedef struct FILE

cdef extern from "drmaa.h":
	cdef enum drmaa_verbose_level_t:
		DRMAA_LOG_ALL,
		DRMAA_LOG_DEBUG_2,
		DRMAA_LOG_DEBUG_1,
		DRMAA_LOG_WARNING,
		DRMAA_LOG_ERROR,
		DRMAA_LOG_NONE
	
	void drmaa_set_logging_output( FILE *file )

	int drmaa_read_configuration_file(
		char *filename,
		char *error_diagnosis, size_t error_diag_len
		)

	int drmaa_read_configuration(
		char *configuration, size_t conf_len,
		char *error_diagnosis, size_t error_diag_len
		)

cdef enum:
	ERRLEN = drmaa_pyrex.DRMAA_ERROR_STRING_BUFFER

def read_configuration_file( filename ):
	cdef int rc
	cdef char errmsg[ ERRLEN ]
	filename = str(filename)
	rc = drmaa_read_configuration_file( filename, errmsg, ERRLEN )
	if rc:
		raise drmaa_pyrex.drmaa_error( rc, errmsg )

def read_configuration( conf ):
	cdef int rc
	cdef char errmsg[ ERRLEN ]
	conf = str(conf)
	rc = drmaa_read_configuration( conf, len(conf), errmsg, ERRLEN )
	if rc:
		raise drmaa_pyrex.drmaa_error( rc, errmsg )

