# $Id: tui.py 529 2007-12-20 21:04:22Z lukasz $
from testing.core import *
from testing import term

import os
import shutil
import sys



# class TUI_test_runner( Test_runner_base ):
# 	def __init__( self, options ):
# 		self.options = options
# 		self.out = sys.stderr
# 
# 	def __call__( self, *args, **kwargs ):
# 		self.run( *args, **kwargs )
# 
# 	def run( self, testsuite ):
# 		#result = self.create_result( testsuite )
# 		if self.options.log_dir:
# 			if os.access( self.options.log_dir, os.F_OK ):
# 				shutil.rmtree( self.options.log_dir )
# 			os.makedirs( self.options.log_dir, 0700 )
# 		Test_runner_base.run( self, testsuite )
# 		#result.beforeTestsuite( testsuite )
# 		#testsuite( result )
# 		#result.afterTestsuite( testsuite )
# 		if self.options.verbosity == 1:
# 			self.out.write( "\n" )
# 		if self.options.verbosity >= 1:
# 			self.out.write(
# 					"="*40 + "\n" + (
# 					"run %(tests_ran)d tests in %(time_spend)s\n"+\
# 					"successful: %(tests_passed)d (%(success_rate)d%%)\n"+\
# 					"errors: %(errors)d\n"+\
# 					"failures: %(failures)d\n") \
# 					% result.status()
# 					)
# 	
# 	def create_result( self, testsuite ):
# 		return _TUI_test_result(
# 				testsuite = testsuite,
# 				options = self.options,
# 				output = self.out,
# 				)


def tui_test_runner( options ):
	def factory( testsuite ):
		return _TUI_test_result( testsuite, options, sys.stderr )
	return Test_runner( factory )


class _TUI_test_result( Test_result_base ):
	def __init__( self, testsuite, options, output ):
		Test_result_base.__init__( self, testsuite )
		self.testsuite = testsuite
		self.options = options
		if self.options.verbosity >= 2:
			self.marks = dict(
					success = "ok",
					error = "ERROR",
					failure = "FAILURE",
					)
		else:
			self.marks = dict( success=".", error="E", failure="F" )
		self.out = output

	def beforeTestsuite( self, testsuite ):
		Test_result_base.beforeTestsuite( self, testsuite )
		if self.options.log_dir:
			if os.access( self.options.log_dir, os.F_OK ):
				shutil.rmtree( self.options.log_dir )
			os.makedirs( self.options.log_dir, 0700 )
	
	def afterTestsuite( self, testsuite ):
		Test_result_base.afterTestsuite( self, testsuite )
		if self.options.verbosity == 1:
			self.out.write( "\n" )
		if self.options.verbosity >= 2 and self.tests_passed < self.tests_ran:
			for test in self.tests:
				result, exception = test.result
				if result != "success":
					self.out.write( "-"*40+"\n" + \
							"%(test)s %(result)s\n%(exception)s" % dict ( \
									test = test,
									result = result,
									exception = exception,
							) )
		if self.options.verbosity >= 2 and self.tests_passed < self.tests_ran:
			self.out.write( "="*40+"\n" + "Tests which do not passed:\n" )
			for test in self.tests:
				result, exception = test.result
				if result != "success":
					self.out.write( "%s\n" % test ) 
		if self.options.verbosity >= 1:
			self.out.write(
					"="*40 + "\n" + (
					"run %(tests_ran)d tests in %(time_spend)s\n"+\
					"successful: %(tests_passed)d (%(success_rate)d%%)\n"+\
					"errors: %(errors)d\n"+\
					"failures: %(failures)d\n") \
					% self.status()
					)

	def startTest( self, test ):
		Test_result_base.startTest( self, test )
		if self.options.verbosity >= 2:
			self.out.write( test_name(test) + " ... " )
		term.set_title( \
				("running test %(test_no)d/%(all_tests)d (%(percent_done)d%%); "+ \
				"success rate: %(success_rate)d%%") \
				% self.status() )
		self.redirections = []
		if self.options.log_dir:
			stdout = osp.join( self.options.log_dir, test.id()+".out" )
			stderr = osp.join( self.options.log_dir, test.id()+".err" )
			self.redirections += [ \
				File_redirect( 1, stdout ), \
				File_redirect( 2, stderr ), \
				]

	def afterTest( self, test, err ):
		Test_result_base.afterTest( self, test, err )
		if err:
			print >>sys.stderr, self._exc_info_to_string( err, test ) + "\n"
		for f in self.redirections:
			f.close()
		self.redirections = []

	def addResult( self, test, result ):
		Test_result_base.addResult( self, test, result )
		if self.options.verbosity == 1:
			self.out.write( self.marks[ result ] )
		elif self.options.verbosity >= 2:
			self.out.write( "(in %(time)s) %(result)s\n" % dict( \
						result = self.marks[ result ],
						time = self.tests[-1].time_spend(),
						) )


